using org.bouncycastle.asn1;

namespace org.bouncycastle.asn1.pkcs
{
    public class SafeBag
        : ASN1Encodable
    {
        DERObjectIdentifier         bagId;
        ASN1Object                   bagValue;
        ASN1Set                     bagAttributes;
    
        public SafeBag(
            DERObjectIdentifier     oid,
            ASN1Object               obj)
        {
            this.bagId = oid;
            this.bagValue = obj;
            this.bagAttributes = null;
        }
    
        public SafeBag(
            DERObjectIdentifier     oid,
            ASN1Object               obj,
            ASN1Set                 bagAttributes)
        {
            this.bagId = oid;
            this.bagValue = obj;
            this.bagAttributes = bagAttributes;
        }
    
        public SafeBag(
            ASN1Sequence    seq)
        {
            this.bagId = (DERObjectIdentifier)seq.getObjectAt(0);
            this.bagValue = ((DERTaggedObject)seq.getObjectAt(1)).getObject();
            if (seq.size() == 3)
            {
                this.bagAttributes = (ASN1Set)seq.getObjectAt(2);
            }
        }
    
        public DERObjectIdentifier getBagId()
        {
            return bagId;
        }
    
        public ASN1Object getBagValue()
        {
            return bagValue;
        }
    
        public ASN1Set getBagAttributes()
        {
            return bagAttributes;
        }
    
        public override ASN1Object toASN1Object()
        {
            ASN1EncodableVector v = new ASN1EncodableVector();
    
            v.add(bagId);
            v.add(new DERTaggedObject(0, bagValue));
    
            if (bagAttributes != null)
            {
                v.add(bagAttributes);
            }
    
            return new DERSequence(v);
        }
    }
}
